"""
Preparses input from the interpreter

Modified input.

  -- All ^'s are replaced by **'s.

  -- If M is a variable and i an integer, then M.i is replaced by M._i,
     so generators can be accessed as in MAGMA.

All other input is processed normally.
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

__author__ = 'William Stein <was@math.harvard.edu>'
__version__= '0.2.0'
__license__ = 'GPL'
__date__   = 'March 2005'

# IPython has a prefilter() function that analyzes each input line. We redefine
# it here to first pre-process certain forms of input

# The prototype of any alternate prefilter must be like this one (the name
# doesn't matter):
# - line is a string containing the user input line.
# - continuation is a parameter which tells us if we are processing a first line of
#   user input or the second or higher of a multi-line statement.

def prefilter_paste(self, line, continuation):
    """
        Alternate prefilter for input.
    """

    from re import match
    from IPython.iplib import InteractiveShell

    # This is so it's OK to have lots of blank space at the
    # beginning of any line. 
    if not continuation:
        line = line.lstrip()
    line = line.rstrip()


    # Get rid of leading >>> so that pasting of examples from the documentation
    # works.  This is like MAGMA's SetLinePrompt(false).
    if line[:3] == ">>>":
        line = line[3:].lstrip()
    elif line[:5] == "sage:":
        line = line[5:].lstrip()

    #################################################################
    # A "load" command, like \r file in PARI or load "file" in MAGMA
    #################################################################
    if line[:5] == 'load ':
        # The -i so the file is run with the same environment,
        # e.g., including the "from sage import *"
        line = '%run -i ' + line[5:]


    # Use "^" for exponentation.
    line = line.replace("\^","___**___")
    line = line.replace("^","**")
    line = line.replace("___**___","^")

    # Generators like in MAGMA: replace all ".<number>" by "._<number>"
    # If . is preceeded by \, then replace "\." by ".".

##     L = line.split(".")
##     line=""
##     for i in range(len(L)-1):
##         if i+1 < len(L) and i+1 >= 0 and\
##                len(L[i+1])>0 and L[i+1][0] >= '0' and L[i+1][0] <= '9' and \
##                not __ends_in_an_integer(L[i]):  # needed to avoid 3.14 --> 3._14
##             if L[i][-1:] == "\\":
##                 line += L[i][:-1] + "."
##             else:
##                 line += L[i] + "._"
##         else:
##             line += L[i] + "."
##     if len(L)>0:
##         line += L[len(L)-1]

    # Time command like in MAGMA:
    if not continuation and line.lstrip()[:5] == "time ":
        if line[-1:] == ";":
            line = line[:-1]
        line = '__time__=misc.cputime(); %s; print "Time:", misc.cputime(__time__), "seconds"'%\
               line.lstrip()[4:]
        
    return InteractiveShell._prefilter(self, line, continuation)


def __ends_in_an_integer(x):
    # return true precisely if the first non-digit is
    # not a letter or "\".
    i=len(x)-1
    while i>=0 and i<len(x) and x[i] >= '0' and x[i] <= '9':
        i-=1
    return i==-1 or not ((x[i].lower()>='a' and x[i].lower()<='z') or x[i] == '\\')

            
# Rebind this to be the new IPython prefilter:
from IPython.iplib import InteractiveShell
InteractiveShell.prefilter = prefilter_paste

# Clean up the namespace.
del InteractiveShell,prefilter_paste



